/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2013  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "anhero.h"
#include <QIcon>
#include <QApplication>
#include <KDE/KGlobal>
#include <KDE/KCrash>
#include <kdeversion.h>
#include <util/util.h>

namespace LeechCraft
{
namespace AnHero
{
	void Plugin::Init (ICoreProxy_ptr)
	{
		Util::InstallTranslator ("anhero");

		if (!QApplication::arguments ().contains ("-noanhero"))
		{
#if KDE_VERSION_MINOR < 6
			KCrash::setApplicationPath (qApp->applicationDirPath ());
#endif
			KCrash::setApplicationName ("LeechCraft");
#if KDE_VERSION_MINOR < 5
			KCrash::setCrashHandler (KCrash::defaultCrashHandler);
#else
			KCrash::setDrKonqiEnabled (true);;
#endif
		}
	}

	void Plugin::SecondInit ()
	{
	}

	void Plugin::Release ()
	{
	}

	QByteArray Plugin::GetUniqueID () const
	{
		return "org.LeechCraft.AnHero";
	}

	QString Plugin::GetName () const
	{
		return "AnHero";
	}

	QString Plugin::GetInfo () const
	{
		return tr ("Crash handler");
	}

	QIcon Plugin::GetIcon () const
	{
		static QIcon icon (":/resources/images/anhero.svg");
		return icon;
	}
}
}

LC_EXPORT_PLUGIN (leechcraft_anhero, LeechCraft::AnHero::Plugin);
