/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2013  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "shooterdialog.h"
#include <QtDebug>

namespace LeechCraft
{
namespace Auscrie
{
	ShooterDialog::ShooterDialog (QWidget *parent)
	: QDialog (parent)
	{
		Ui_.setupUi (this);
		on_Format__currentIndexChanged (Ui_.Format_->currentText ());
	}

	ShooterDialog::Action ShooterDialog::GetAction () const
	{
		switch (Ui_.ActionBox_->currentIndex ())
		{
		case 0:
		case 1:
		case 2:
			return Action::Upload;
		case 3:
			return Action::Save;
		default:
			qWarning () << Q_FUNC_INFO
					<< Ui_.ActionBox_->currentIndex ()
					<< "unhandled";
			return Action::Save;
		}
	}

	ShooterDialog::Mode ShooterDialog::GetMode () const
	{
		switch (Ui_.ModeBox_->currentIndex ())
		{
		case 0:
			return Mode::LCWindowOverlay;
		case 1:
			return Mode::LCWindow;
		case 2:
			return Mode::CurrentScreen;
		case 3:
			return Mode::WholeDesktop;
		default:
			qWarning () << Q_FUNC_INFO
					<< Ui_.ModeBox_->currentIndex ()
					<< "unhandled";
			return Mode::LCWindowOverlay;
		}
	}

	bool ShooterDialog::ShouldHide () const
	{
		return Ui_.HideThis_->checkState () == Qt::Checked;
	}

	Poster::HostingService ShooterDialog::GetHostingService () const
	{
		switch (Ui_.ActionBox_->currentIndex ())
		{
		case 0:
			return Poster::DumpBitcheeseNet;
		case 1:
			return Poster::SavepicRu;
		case 2:
			return Poster::ImagebinCa;
		default:
			qWarning () << Q_FUNC_INFO
					<< Ui_.ActionBox_->currentIndex ()
					<< "unhandled, defaulting to imagebin.ca";
			return Poster::ImagebinCa;
		}
	}

	int ShooterDialog::GetTimeout () const
	{
		return Ui_.Timeout_->value ();
	}

	QString ShooterDialog::GetFormat () const
	{
		return Ui_.Format_->currentText ();
	}

	int ShooterDialog::GetQuality () const
	{
		const int val = Ui_.QualityBox_->value ();
		return Ui_.Format_->currentText () == "JPG" ?
				val :
				100 - val;
	}

	void ShooterDialog::on_Format__currentIndexChanged (const QString& str)
	{
		if (str == "JPG")
			Ui_.SettingLabel_->setText ("Quality:");
		else
			Ui_.SettingLabel_->setText ("Compression:");
	}
}
}
